@@ -403,11 +403,6 @@ class _SSLContext(object):
         self._password = password
 
     def wrap_socket(self, sock, **kwargs):
-        # If we are on Python 2 the return value from socket.fromfd
-        # is an internal socket object but what we need for ssl wrap
-        # is the wrapper around it :(
-        if PY2 and not isinstance(sock, socket.socket):
-            sock = socket.socket(sock.family, sock.type, sock.proto, sock)
         return ssl.wrap_socket(sock, keyfile=self._keyfile,
                                certfile=self._certfile,
                                ssl_version=self._protocol, **kwargs)
@@ -482,8 +477,13 @@ class BaseWSGIServer(HTTPServer, object):
                 ssl_context = load_ssl_context(*ssl_context)
             if ssl_context == 'adhoc':
                 ssl_context = generate_adhoc_ssl_context()
-            self.socket = ssl_context.wrap_socket(self.socket,
-                                                  server_side=True)
+            # If we are on Python 2 the return value from socket.fromfd
+            # is an internal socket object but what we need for ssl wrap
+            # is the wrapper around it :(
+            sock = self.socket
+            if PY2 and not isinstance(sock, socket.socket):
+                sock = socket.socket(sock.family, sock.type, sock.proto, sock)
+            self.socket = ssl_context.wrap_socket(sock, server_side=True)
             self.ssl_context = ssl_context
         else:
             self.ssl_context = None
@@ -494,14 +494,6 @@ class BaseWSGIServer(HTTPServer, object):
     def serve_forever(self):
         self.shutdown_signal = False
         try:
-            if os.environ.get('WERKZEUG_RUN_MAIN') != 'true':
-                display_hostname = self.host != '*' and self.host or 'localhost'
-                if ':' in display_hostname:
-                    display_hostname = '[%s]' % display_hostname
-                quit_msg = '(Press CTRL+C to quit)'
-                _log('info', ' * Running on %s://%s:%d/ %s',
-                     self.ssl_context is None and 'http' or 'https',
-                     display_hostname, self.port, quit_msg)
             HTTPServer.serve_forever(self)
         except KeyboardInterrupt:
             pass
@@ -511,8 +503,7 @@ class BaseWSGIServer(HTTPServer, object):
     def handle_error(self, request, client_address):
         if self.passthrough_errors:
             raise
-        else:
-            return HTTPServer.handle_error(self, request, client_address)
+        return HTTPServer.handle_error(self, request, client_address)
 
     def get_request(self):
         con, info = self.socket.accept()
@@ -642,15 +633,28 @@ def run_simple(hostname, port, application, use_reloader=False,
         from werkzeug.wsgi import SharedDataMiddleware
         application = SharedDataMiddleware(application, static_files)
 
+    def log_startup(sock):
+        display_hostname = hostname not in ('', '*') and hostname or 'localhost'
+        if ':' in display_hostname:
+            display_hostname = '[%s]' % display_hostname
+        quit_msg = '(Press CTRL+C to quit)'
+        port = sock.getsockname()[1]
+        _log('info', ' * Running on %s://%s:%d/ %s',
+             ssl_context is None and 'http' or 'https',
+             display_hostname, port, quit_msg)
+
     def inner():
         try:
             fd = int(os.environ['WERKZEUG_SERVER_FD'])
         except (LookupError, ValueError):
             fd = None
-        make_server(hostname, port, application, threaded,
-                    processes, request_handler,
-                    passthrough_errors, ssl_context,
-                    fd=fd).serve_forever()
+        srv = make_server(hostname, port, application, threaded,
+                          processes, request_handler,
+                          passthrough_errors, ssl_context,
+                          fd=fd)
+        if fd is None:
+            log_startup(srv.socket)
+        srv.serve_forever()
 
     if use_reloader:
         # If we're not running already in the subprocess that is the
@@ -677,6 +681,7 @@ def run_simple(hostname, port, application, use_reloader=False,
             if can_open_by_fd:
                 os.environ['WERKZEUG_SERVER_FD'] = str(s.fileno())
                 s.listen(LISTEN_QUEUE)
+                log_startup(s)
             else:
                 s.close()
 
